#include "SonyIR.h"
//***************************************************************************
// IRPropClock.c  by Otis Irace  Oct 28, 2012
// Uses SONY IR Remote to Set Time and Date.
// P16F648A @8Mhz  RA0 = Index Pulse,  RA1 = Blue LED, RA3 = Time Set PB,
// PORTB = Display LEDS.
// Font Set created using Excel Spreadsheet.
// Upper Half of Clock Display shows Characters Backwards,  the user must
// correct when using any Display Routine.
// Use Centermost PB for RA3 to enable Time Set Mode.
// Timing is based on code from Roman Black, Bresenham's Algorithm
// and loop by Leland Durrette
// IR Remote Keyboard Map:
// 2 =  Sets Minutes
// 3 =  Sets Hours
// 4 =  Sets Days
// 5 =  Sets Months
// 6 =  Sets Years
// 9 = Returns to Clock Mode
//***************************************************************************
//Variable Declarations
#define index_pulse PORTA.F0
#define timeset PORTA.F3
#define CNT_THRESHOLD 1000000
#define CNT_RATE 256
//***************************************************************************
//                          1234567890123456
//***************************************************************************
unsigned char const monthname[] = {
         ' ', ' ', ' ',
         'J', 'A', 'N',
         'F', 'E', 'B',
         'M', 'A', 'R',
         'A', 'P', 'R',
         'M', 'A', 'Y',
         'J', 'U', 'N',
         'J', 'U', 'L',
         'A', 'U', 'G',
         'S', 'E', 'P',
         'O', 'C', 'T',
         'N', 'O', 'V',
         'D', 'E', 'C' };
unsigned char const font[] = {
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF,  // sp   1
    0xFF, 0xFF, 0x82, 0xFF, 0xFF,  // !    2
    0xFF, 0xFF, 0x9F, 0xFF, 0x9F,  // "    3
    0xEB, 0x80, 0xEB, 0x80, 0xEB,  // #    4
    0xD9, 0x94, 0xD5, 0x94, 0xCD,  // $    5
    0xDC, 0xEC, 0xF7, 0x9B, 0x9D,  // %    6
    0xFA, 0xDD, 0xAA, 0xB6, 0xC9,  // &    7
    0xFF, 0xFF, 0x9F, 0xAF, 0xFF,  // '    8
    0xFF, 0xBE, 0xDD, 0xE3, 0xFF,  // (    9
    0xFF, 0xE3, 0xDD, 0xBE, 0xFF,  // )    10
    0xEB, 0xF7, 0xC1, 0xF7, 0xEB,  // *    11
    0xF7, 0xF7, 0xC1, 0xF7, 0xF7,  // +    12
    0xFF, 0xFF, 0xF9, 0xFA, 0xFF,  // ,    13
    0xF7, 0xF7, 0xF7, 0xF7, 0xF7,  // -    14
    0xFF, 0xFC, 0xFC, 0xFF, 0xFF,  // .    15
    0xDF, 0xEF, 0xF7, 0xFB, 0xFD,  // /    16
    0xC1, 0xBE, 0xBE, 0xBE, 0xC1,  // 0    17
    0xFF, 0xFE, 0x80, 0xDE, 0xFF,  // 1    18
    0xCE, 0xB6, 0xBA, 0xBC, 0xDE,  // 2    19
    0xB9, 0x96, 0xAE, 0xBE, 0xBD,  // 3    20
    0xFB, 0x80, 0xDB, 0xEB, 0xF3,  // 4    21
    0xB1, 0xAE, 0xAE, 0xAE, 0x8D,  // 5    22
    0xF9, 0xB6, 0xB6, 0xD6, 0xE1,  // 6    23
    0x8F, 0xB7, 0xB8, 0xBF, 0x9F,  // 7    24
    0xC9, 0xB6, 0xB6, 0xB6, 0xC9,  // 8    25
    0xC3, 0xB5, 0xB6, 0xB6, 0xCF,  // 9    26
    0xFF, 0xFF, 0xC9, 0xC9, 0xFF,  // :    27
    0xFF, 0xC9, 0xCA, 0xFF, 0xFF,  // ;    28
    0xFF, 0xBE, 0xDD, 0xEB, 0xF7,  // <    29
    0xEB, 0xEB, 0xEB, 0xEB, 0xEB,  // =    30
    0xF7, 0xEB, 0xDD, 0xBE, 0xFF,  // >    31
    0xCF, 0xB7, 0xBA, 0xBF, 0xDF,  // ?    32
    0xC1, 0xBE, 0xB0, 0xB6, 0xD9,  // @    33
    0xC0, 0xB7, 0xB7, 0xB7, 0xC0,  // A    34
    0xC9, 0xB6, 0xB6, 0xB6, 0x80,  // B    35
    0xDD, 0xBE, 0xBE, 0xBE, 0xC1,  // C    36
    0xC1, 0xBE, 0xBE, 0xBE, 0x80,  // D    37
    0xBE, 0xB6, 0xB6, 0xB6, 0x80,  // E    38
    0xBF, 0xB7, 0xB7, 0xB7, 0x80,  // F    39
    0xD0, 0xB6, 0xB6, 0xBE, 0xC1,  // G    40
    0x80, 0xF7, 0xF7, 0xF7, 0x80,  // H    41
    0xBE, 0xBE, 0x80, 0xBE, 0xBE,  // I    42
    0x81, 0xFE, 0xFE, 0xFE, 0xF9,  // J    43
    0xBE, 0xDD, 0xEB, 0xF7, 0x80,  // K    44
    0xFE, 0xFE, 0xFE, 0xFE, 0x80,  // L    45
    0x80, 0xCF, 0xE7, 0xCF, 0x80,  // M    46
    0x80, 0xF9, 0xF7, 0xCF, 0x80,  // N    47
    0x80, 0xBE, 0xBE, 0xBE, 0x80,  // O    48
    0x87, 0xB7, 0xB7, 0xB7, 0x80,  // P    49
    0x81, 0xBC, 0xB9, 0xBD, 0x81,  // Q    50
    0xCE, 0xB5, 0xB3, 0xB7, 0x80,  // R    51
    0xD0, 0xB6, 0xB6, 0xB6, 0x85,  // S    52
    0xBF, 0xBF, 0x80, 0xBF, 0xBF,  // T    53
    0x81, 0xFE, 0xFE, 0xFE, 0x81,  // U    54
    0x83, 0xFD, 0xFE, 0xFD, 0x83,  // V    55
    0x81, 0xFE, 0xF1, 0xFE, 0x81,  // W    56
    0x9C, 0xEB, 0xF7, 0xEB, 0x9C,  // X    57
    0x9F, 0xEF, 0xF0, 0xEF, 0x9F,  // Y    58
    0x9E, 0xAE, 0xB6, 0xBA, 0xBC,  // m    59
    0xFF, 0xFF, 0xBE, 0xBE, 0x80,  // [    60
    0xAB, 0xCB, 0xE0, 0xCB, 0xAB,  // 55   61
    0xFF, 0x80, 0xBE, 0xBE, 0xFF,  // ]    62
    0xEF, 0xDF, 0xBF, 0xDF, 0xEF,  // ^    63
    0xFE, 0xFE, 0xFE, 0xFE, 0xFE,  // _    64
    0xFF, 0xEF, 0xDF, 0xBF, 0xFF,  // '    65
    0xF0, 0xEA, 0xEA, 0xEA, 0xFD,  // a    66
    0xF1, 0xEE, 0xEE, 0xF6, 0x80,  // b    67
    0xFD, 0xEE, 0xEE, 0xEE, 0xF1,  // c    68
    0x80, 0xF6, 0xEE, 0xEE, 0xF1,  // d    69
    0xF3, 0xEA, 0xEA, 0xEA, 0xF1,  // e    70
    0xDF, 0xBF, 0xB7, 0xC0, 0xF7,  // f    71
    0xC1, 0xDA, 0xDA, 0xDA, 0xE7,  // g    72
    0xF8, 0xF7, 0xF7, 0xFB, 0x80,  // h    73
    0xFF, 0xFE, 0xA0, 0xEE, 0xFF,  // i    74
    0xFF, 0xA1, 0xEE, 0xFE, 0xFD,  // j    75
    0xFF, 0xEE, 0xF5, 0xFB, 0x80,  // k    76
    0xFF, 0xFE, 0x80, 0xBE, 0xFF,  // l    77
    0xF8, 0xF7, 0xF9, 0xF7, 0xF0,  // m    78
    0xF8, 0xF7, 0xF7, 0xFB, 0xF0,  // n    79
    0xF9, 0xF6, 0xF6, 0xF6, 0xF9,  // o    80
    0xF7, 0xEB, 0xEB, 0xE0, 0xFF,  // p    81
    0xE0, 0xF3, 0xEB, 0xEB, 0xF7,  // q    82
    0xF7, 0xEF, 0xEF, 0xF7, 0xE0,  // r    83
    0xFD, 0xEA, 0xEA, 0xEA, 0xF6,  // s    84
    0xFD, 0xFE, 0xDE, 0x81, 0xDF,  // t    85
    0xF0, 0xFD, 0xFE, 0xFE, 0xF1,  // u    86
    0xF3, 0xFD, 0xFE, 0xFD, 0xF3,  // v    87
    0xF1, 0xFE, 0xF9, 0xFE, 0xF1,  // w    88
    0xEE, 0xF5, 0xFB, 0xF5, 0xEE,  // x    89
    0xE1, 0xFA, 0xFA, 0xFA, 0xE7,  // y    90
    0xEE, 0xE6, 0xEA, 0xEC, 0xEE}; // m    91

char *Message1 = " MIN";
char *Message2 = " HRS";
char *Message3 = " DAY";
char *Message4 = " MONTH";
char *Message5 = " YEAR";
char *Message6 = " ";
char *Message7 = " ";
const refresh = 10;
const char shiftspd = 3;
const sdelay = 100;   //Row Width  was 200
const spaced = 50;  // was 1000
char tempbit[8], var, chrspace;
unsigned long cnt;
unsigned latch;
unsigned char n, j, m;
int k, charpos, delayscroll;
unsigned char sony_rx_cmd, sony_rx_addr;
struct TTime{ char year, month, day, hours, minutes,
seconds, twentyfour;} TimeRead;
//**************************************************************************
void Display(char place)
{
char i;
    place = place - 1;
    PORTB = 0xFF;
    Delay_us(300);
    for(i = 0; i <= 4; i++)
    {
        PORTB = font[(((place - 31) * 5) + i)];
        Delay_us(sdelay);
    }
    PORTB = 0xFF;
    Delay_us(300);
}
//**************************************************************************
void iDisplay(char place)
{
char i;
    place = place - 1;
    PORTB = 0xFF;
    Delay_us(300);
    for(i = 4; i <= 4; i--)
        {
        var = font[(((place - 31) * 5) + i)];
        tempbit[0] = var.F0;
        tempbit[1] = var.F1;
        tempbit[2] = var.F2;
        tempbit[3] = var.F3;
        tempbit[4] = var.F4;
        tempbit[5] = var.F5;
        tempbit[6] = var.F6;
        tempbit[7] = var.F7;
        var.F7 =  1;
        var.F6 =  tempbit[0];
        var.F5 =  tempbit[1];
        var.F4 =  tempbit[2];
        var.F3 =  tempbit[3];
        var.F2 =  tempbit[4];
        var.F1 =  tempbit[5];
        var.F0 =  tempbit[6];
    PORTB = var;
    Delay_us(sdelay);
        }
    PORTB = 0xFF;
    Delay_us(300);
}
//**************************************************************************
void LEDString(char s[])
//Substitutes Message_x into const char s[]
{
	char i;
	for (i = 0; s[i] != 0; i++)		  // 	Display String Message
	{
    Display(s[i]);
	}
}// End LEDString
//**************************************************************************
void space(void)
{
PORTB = 0b11111111;
Delay_us(spaced);
} //**************************************************************************
void digitset(void)
{
     if (TimeRead.seconds == 60)
        {
        Timeread.seconds = 0;
        ++Timeread.minutes;
        }
     if (TimeRead.minutes == 60)
        {
        Timeread.minutes = 0;
        ++Timeread.hours;
        ++Timeread.twentyfour;
        }
     if (TimeRead.twentyfour >= 12)
        {

        }
     if (TimeRead.hours == 13)
        {
        Timeread.hours = 1;
        }
     if (Timeread.twentyfour >= 24)
        {
        Timeread.twentyfour = 1;
        ++Timeread.day;
        }
     if (TimeRead.day >= 32)
        {
        Timeread.day = 1;
        ++TimeRead.month;
        }
     if (TimeRead.month >= 13)
        {
        ++Timeread.year;
        Timeread.month = 1;
        }
     if (TimeRead.year == 99)
        {
        Timeread.year = 0;
        }
}
//**************************************************************************
void DisplayTimeSet()
{
  for(k = 0; k <= 10; k++)
    {
    for(m = 0; m <= refresh; m++)
      {
      while (PORTA.F0 == 1)       // Wait here until Index Trigger
       {
       }
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display('E');
       Display('M');
       Display('I');
       Display('T');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       iDisplay('S');
       iDisplay('E');
       iDisplay('T');
       space();
       }
    }
}
//**************************************************************************
void DisplayMessage(char *text, char shiftspeed, char hold)
{
 for(charpos = 1; charpos <= strlen(text); ++charpos)// Loop until Message done
    {
    if (charpos >= strlen(text))
       {
       for (n = hold; n >= 0; n--)
         {
         while (index_pulse == 1)  // Wait for Index Pulse
           {
           }
           for(k = charpos -1; k >= 0; k--) // Display one Char, then 2, then 3...
     	       {
     	       Display(text[k]);
             }
             if (n <= 0) return;  // If Line Hold is mero, then Return
         }  // End For hold
       } // End If Charpos

    for(delayscroll = 0; delayscroll <= shiftspeed; delayscroll++)
      {
      while (index_pulse == 1)  // Wait for Index Pulse
         {
         }
         for(k = charpos; k >= 0; k--) // Display one Char, then 2, then 3...
     	     {
     	     Display(text[k]);
           }
      } // End For delayscroll
    } // End For charpos
} // End DisplayMessage
//***************************************************************************

void interrupt()
{
  if (INTCON.T0IF == 1)                // once per iteration of timer
  {
    cnt += CNT_RATE;                   // build a count of iterations
    if (cnt > CNT_THRESHOLD)           // when a second has passed,
    {
      cnt -= CNT_THRESHOLD;            // accumulating minuscule error
      Timeread.seconds++;              // Increment Seconds
      timeset = ~timeset;              // Toggle PORTB LEDs
      PORTA.F1 = ~PORTA.F1;            // Toggle Blue LED
    }
    INTCON.T0IF = 0;                   // Clear TMR0 Flag
  }
  return;
}
//***************************************************************************
void main()
{
  unsigned char code, last;
  OPTION_REG = 0;
  INTCON.GIE = 1;           // Enable All Unmasked Interrupts
  INTCON.T0IE = 1;          // TMR0 Overflow Interrupt Enable bit
  CMCON =      0b00000111;
	PORTA =      0b00000000;
	PORTB =      0b00000000;
  TRISA =      0b00001101;
	TRISB =      0b00000000;
  TMR0 = 0;                 // Timer0 initial value
  PORTA.F4 = 1;             // Turn Blue LED ON
  n = 0;
  cnt = 0;
  Timeread.seconds = 00;
  Timeread.minutes = 00;
  Timeread.hours = 12;
  Timeread.twentyfour = 00;
  Timeread.day = 1;
  Timeread.month = 1;
  Timeread.year= 12;
  latch = 0;
//***************************************************************************
while(1 == 1)
   {
      while (index_pulse == 1)       // Wait here until Index Trigger
       {
       }
       if (timeset == 0)             // Go to Time Set Mode
          {
          latch = latch ^ 1;
          DisplayTimeSet();          // Call Display "Time Set"
          }
       if (latch == 0)               // Return Back to Clock Mode
          {
          INTCON.T0IE = 1;
          n = 99;
          }
        if (latch == 1)              // If Time Set Mode, Read Sony Commands
          {
          INTCON.T0IE = 0;
          n = sony_rx_run(code);
          }

switch(n)
		{
//***************************************************************************
    case 1:  // Set Minutes
    ++Timeread.minutes;
    for(m = 0; m <= refresh; m++)
    {
      while (index_pulse == 1)       // Wait here until Index Trigger
       {
       }
       digitset();
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display((TimeRead.minutes % 10) + 48);
       Display((TimeRead.minutes / 10) + 48);
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       iDisplay('M');
       iDisplay('I');
       iDisplay('N');
       space();
    }
    break;
//***************************************************************************
		case 2:  // Set Hours
       ++Timeread.hours;
       ++Timeread.twentyfour;
    for(m = 0; m <= refresh; m++)
    {
      while (index_pulse == 1)       // Wait here until Index Trigger
       {
       }
       digitset();
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display((TimeRead.hours % 10) + 48);
       Display((TimeRead.hours / 10) + 48);
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       iDisplay('H');
       iDisplay('R');
       iDisplay('S');
       space();
    }
    break;
//***************************************************************************
		case 8:

    n = 99;
    latch = 0;

    break;
//***************************************************************************
		case 3:  // Set Days
       ++Timeread.day;
    for(m = 0; m <= refresh; m++)
    {
      while (index_pulse == 1)       // Wait here until Index Trigger
       {
       }
       digitset();
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display((TimeRead.day % 10) + 48);
       Display((TimeRead.day / 10) + 48);
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       iDisplay('D');
       iDisplay('A');
       iDisplay('Y');
       iDisplay('S');
       space();
    }
    break;
//***************************************************************************
		case 4:  // Set Months
       ++Timeread.month;
    for(m = 0; m <= refresh; m++)
    {
      while (index_pulse == 1)       // Wait here until Index Trigger
       {
       }
       digitset();
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
//       for(i = 4; i <= 4; i--)
       for (j = 2; j <= 2; j--)
           {
           chrspace = monthname[((Timeread.month * 3) + j)];
           Display(chrspace);
           }

       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       iDisplay('M');
       iDisplay('O');
       iDisplay('N');
       iDisplay('T');
       iDisplay('H');
       space();
    }
    break;
//***************************************************************************
		case 5:  // Set Years
       ++Timeread.year;
    for(m = 0; m <= refresh; m++)
    {
      while (index_pulse == 1)       // Wait here until Index Trigger
       {
       }

       digitset();
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display((TimeRead.year % 10) + 48);
       Display((TimeRead.year / 10) + 48);
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       Display(' ');
       iDisplay('Y');
       iDisplay('E');
       iDisplay('A');
       iDisplay('R');
       space();
    }
    break;
//***************************************************************************
    case 99:        // Display Time and Date
       digitset();
       space();
       space();
       Display(' ');
       Display((TimeRead.seconds % 10) + 48);
       Display((TimeRead.seconds / 10) + 48);
       Display(58);
       Display((TimeRead.minutes % 10) + 48);
       Display((TimeRead.minutes / 10) + 48);
       Display(58);
       Display((TimeRead.hours % 10) + 48);
       Display((TimeRead.hours / 10) + 48);
       space();
       space();
       Display(' ');
       Display(' ');
       Display(' ');

       for (j = 0; j < 3; j++)
           {
           chrspace = monthname[((Timeread.month * 3) + j)];
           iDisplay(chrspace);
           }

       iDisplay(45);
       iDisplay((TimeRead.day / 10) + 48);
       iDisplay((TimeRead.day % 10) + 48);
       iDisplay(45);
       iDisplay((TimeRead.year / 10) + 48);
       iDisplay((TimeRead.year % 10) + 48);
       space();
    break;
//***************************************************************************
    default:
    m = 0;
      } // End switch
    }
} // End Main
//*************************************************************************
